<?php

namespace forge12\contactform7\CF7Captcha {

	use forge12\contactform7\CF7Captcha\core\log\Log_Item;
	use forge12\contactform7\CF7Captcha\core\log\Log_WordPress;

	if ( ! defined( 'ABSPATH' ) ) {
		exit;
	}

	/**
	 * Class AvadaValidator
	 */
	class AvadaValidator {
		public function __construct() {
			add_filter( 'fusion_element_form_content', [ $this, 'addSpamProtection' ], 10, 2 );
			add_filter( 'fusion_form_demo_mode', [ $this, 'validateSpamProtection' ], 10, 1 );
			add_filter( 'fusion_element_contact_form_content', [ $this, 'addSpamProtectionToContactTemplate' ], 10, 2 );
			add_filter( 'init', [ $this, 'validateSpamProtectionFromContactTemplate' ] );
		}

		/**
		 * Get the field name for the post data
		 *
		 * @return string The field name
		 */
		private function getPostFieldName() {
			$settings = CF7Captcha::getInstance()->getSettings();

			// Set the default field name
			$fieldname = $settings['avada']['protect_avada_fieldname'] ?? 'f12_captcha';

			return $fieldname;
		}

		/**
		 * Add spam protection to the contact template.
		 *
		 * @param string $html The original HTML content of the contact template.
		 * @param array  $args Additional arguments.
		 *
		 * @return string The modified HTML content with spam protection added.
		 */
		public function addSpamProtectionToContactTemplate( $html, $args ) {
			$settings  = CF7Captcha::getInstance()->getSettings();
			$fieldname = $this->getPostFieldName();

			$captcha = CaptchaHoneypotGenerator::get_form_field( $fieldname );

			$position = isset( $settings['avada'] ) && isset( $settings['protect_avada_position'] )
				? $settings['avada']['protect_avada_position']
				: 'after_submit';

			if ( $position === 'before_submit' ) {
				$html = str_replace( '<div id="comment-submit-container">', $captcha . '<div id="comment-submit-container">', $html );
			} else {
				$html = str_replace( "</form>", $captcha . '</form>', $html );
			}

			return $html;
		}

		/**
		 * Validate spam protection for the contact template.
		 *
		 * If the submitted form does not meet the spam protection criteria,
		 * log the event and display an error message.
		 *
		 * @return void
		 */
		public function validateSpamProtectionFromContactTemplate() {
			// Check if required form fields are present
			if (
				! isset( $_POST['submit'] ) ||
				! isset( $_POST[ $this->getPostFieldName() ] ) ||
				! isset( $_POST['msg'] ) ||
				! isset( $_POST['contact_name'] )
			) {
				return;
			}

			// Check if the spam protection field is empty
			if ( empty( $_POST[ $this->getPostFieldName() ] ) ) {
				return;
			}

			/*
			 * Add Log Entries
			 */
			$logItem = new Log_Item(
				__( 'Avada Form - Captcha Validator', 'captcha-for-contact-form-7' ),
				$_POST,
				'spam',
				'Captcha failed in AvadaValidator.class.php for Contact Template'
			);
			Log_WordPress::store( $logItem );

			wp_die( __( 'Sorry, this mail has been blocked by spam protection.', 'captcha-for-contact-form-7' ) );
		}

		/**
		 * Add spam protection field to the form.
		 *
		 * Generates the appropriate spam protection field based on the configured captcha method.
		 * Inserts the spam protection field into the form HTML at the specified position.
		 *
		 * @param string $html The original form HTML
		 * @param array $args Additional arguments
		 * @return string The modified form HTML
		 */
		public function addSpamProtection($html, $args)
		{
			$settings = CF7Captcha::getInstance()->getSettings();
			$fieldname = $this->getPostFieldName();

			switch (ControllerAvada::getCaptchaMethod()) {
				case 'math':
					$captcha = CaptchaMathGenerator::get_form_field($fieldname, [
						'classes' => 'fusion-form-input',
						'wrapper_classes' => 'fusion-form-field fusion-form-text-field fusion-form-label-above',
						'attributes' => ['data-form-id' => (int)$args['form_post_id']]
					]);
					break;
				case 'image':
					$captcha = CaptchaImageGenerator::get_form_field($fieldname, [
						'classes' => 'fusion-form-input',
						'wrapper_classes' => 'fusion-form-field fusion-form-text-field fusion-form-label-above',
						'wrapper_attributes' => ['data-form-id' => (int)$args['form_post_id']],
						'attributes' => ['required' => 'true', 'aria-required' => 'true']
					]);
					break;
				default:
					$captcha = CaptchaHoneypotGenerator::get_form_field($fieldname);
					break;
			}

			if (isset($settings['avada']) && isset($settings['avada']['protect_avada_position'])) {
				$position = $settings['avada']['protect_avada_position'];
			} else {
				$position = 'after_submit';
			}

			if ($position === 'before_submit') {
				$html = str_replace('<div class="fusion-form-field fusion-form-submit-field', $captcha . '<div class="fusion-form-field fusion-form-submit-field', $html);
			} else {
				$html = str_replace("</form>", $captcha . '</form>', $html);
			}

			return $html;
		}

		/**
		 * Validate spam protection for the form submission.
		 *
		 * This function checks the submitted form data for the presence and validity of the captcha field.
		 * It performs the necessary validation based on the configured captcha method.
		 * If the captcha is not found or fails validation, it logs the failure and returns an error message.
		 *
		 * @param mixed $value The original value
		 * @return mixed The validated value
		 */
		public function validateSpamProtection($value)
		{
			// Check if the form data is present
			if (!isset($_POST['formData'])) {
				return $value;
			}

			$formData = ControllerAvada::formDataToArray($_POST['formData']);
			$fieldname = $this->getPostFieldName();

			// Check if the field exists
			if (!isset($formData[$fieldname])) {
				// Add Log Entries
				$logItem = new Log_Item(
					__('Avada Form - Captcha Validator', 'captcha-for-contact-form-7'),
					$_POST,
					'spam',
					'Captcha failed in AvadaValidator.class.php, captcha not found.'
				);
				Log_WordPress::store($logItem);

				die(wp_json_encode(ControllerAvada::get_results_from_message('error', 'spam-0')));
			}

			// Check if the Captcha is valid
			switch (ControllerAvada::getCaptchaMethod()) {
				case 'math':
					$isValid = CaptchaMathGenerator::validate($formData[$fieldname], $formData[$fieldname . '_hash']);
					break;
				case 'image':
					$isValid = CaptchaImageGenerator::validate($formData[$fieldname], $formData[$fieldname . '_hash']);
					break;
				default:
					$isValid = CaptchaHoneypotGenerator::validate($formData[$fieldname]);
					break;
			}

			if (!$isValid) {
				// Add Log Entries
				$logItem = new Log_Item(
					__('Avada Form - Captcha Validator', 'captcha-for-contact-form-7'),
					$_POST,
					'spam',
					'Captcha validation failed in AvadaValidator.class.php'
				);
				Log_WordPress::store($logItem);

				die(wp_json_encode(ControllerAvada::get_results_from_message('error', 'spam-1')));
			}

			return $value;
		}
	}
}