<?php

namespace forge12\contactform7\CF7Captcha {

	use forge12\contactform7\CF7Captcha\core\log\Log_Item;
	use forge12\contactform7\CF7Captcha\core\log\Log_WordPress;

	if ( ! defined( 'ABSPATH' ) ) {
		exit;
	}

	/**
	 * Class ControllerAvada
	 */
	class ControllerAvada {
		/**
		 * Admin constructor.
		 */
		public function __construct() {
			if ( function_exists( 'Avada' ) ) {
				$this->init();
			}
		}

		/**
		 * Initialize Avada settings and validators
		 */
		public function init() {
			// Load AvadaRuleValidator
			require_once( 'AvadaRuleValidator.class.php' );
			$RuleValidator = AvadaRuleValidator::getInstance();

			// Check if Captcha is enabled
			if ( $this->isCaptchaEnabled() ) {
				// Load AvadaValidator
				require_once( 'AvadaValidator.class.php' );
				$AV = new AvadaValidator();

				// Check if browser validation is enabled
				if($this->is_browser_validation_enabled()) {
					// Load Avada_Browser_Validator
					require_once( 'Avada_Browser_Validator.php' );
					$Browser_Validator = Avada_Browser_Validator::get_instance();
				}

				// Check if JavaScript validation is enabled
				if($this->is_javascript_validation_enabled()) {
					// Load Avada_Javascript_Validator
					require_once( 'Avada_Javascript_Validator.php' );
					$Javascript_Validator = Avada_Javascript_Validator::get_instance();
				}
			}

			// Check if TimeValidator is enabled
			if ( $this->isTimeValidatorEnabled() ) {
				// Load TimerValidatorAvada
				require_once( 'TimerValidatorAvada.class.php' );
				$TimerValidator = TimerValidatorAvada::getInstance();
			}

			// Check if MultipleSubmissionProtection is enabled
			if ( $this->isMultipleSubmissionProtectionEnabled() ) {
				// Load AvadaMultipleSubmissionProtection
				require_once( 'AvadaMultipleSubmissionProtection.class.php' );
				$MultipleSubmissionProtection = AvadaMultipleSubmissionProtection::getInstance();
			}

			// Check if IPValidator is enabled
			if ( $this->isIPValidatorEnabled() ) {
				// Load AvadaIPLog
				require_once( 'AvadaIPLog.class.php' );
				$AIL = new AvadaIPLog();
			}

			// Register assets
			add_action( 'wp_enqueue_scripts', array( $this, 'addAssets' ) );

			// Filter for form mail arguments after validation
			add_filter( 'fusion_form_send_mail_args', array( $this, 'after_validation' ), 999999999999, 3 );
		}

		/**
		 * Handle actions after form validation
		 *
		 * @param array  $args    Mail arguments
		 * @param string $form_id Form ID
		 * @param string $data    Form data
		 *
		 * @return array Modified mail arguments
		 */
		public function after_validation( $args, $form_id, $data ) {
			/*
			 * Add Log Entries
			 */
			$Log_Item = new Log_Item(
				__( 'Avada Form', 'captcha-for-contact-form-7' ),
				$data,
				'verified',
				'Avada Form successfully committed' );
			Log_WordPress::store( $Log_Item );

			return $args;
		}

		/**
		 * Add assets for Avada form
		 */
		public function addAssets() {
			wp_enqueue_script( 'f12-cf7-captcha-avada', plugin_dir_url( __FILE__ ) . 'assets/f12-cf7-captcha-avada.js', array( 'jquery' ) );
			wp_localize_script( 'f12-cf7-captcha-avada', 'f12_cf7_captcha_avada', array(
				'ajaxurl' => admin_url( 'admin-ajax.php' )
			) );
		}

		/**
		 * Get results from a message
		 *
		 * @param string $type Result status
		 * @param string $info Result info
		 *
		 * @return array Result array
		 */
		public static function get_results_from_message( $type, $info ) {
			return [
				'status' => $type,
				'info'   => $info,
			];
		}

		/**
		 * Check if JavaScript Validation is enabled
		 *
		 * @return bool Whether JavaScript validation is enabled
		 */
		private function is_javascript_validation_enabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect', 'javascript' );
		}

		/**
		 * Check if Browser Validation is enabled
		 *
		 * @return bool Whether Browser validation is enabled
		 */
		private function is_browser_validation_enabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect', 'browser' );
		}

		/**
		 * Convert form data string to an array
		 *
		 * @param string $data Form data string
		 *
		 * @return array Form data as an array
		 */
		public static function formDataToArray( $data ) {
			$data = wp_unslash( $data ); // phpcs:ignore WordPress.Security.NonceVerification, WordPress.Security.ValidatedSanitizedInput
			parse_str( $data, $value );

			return $value;
		}

		/**
		 * Check if IP Validator is enabled
		 *
		 * @return bool Whether IP Validator is enabled
		 */
		private function isIPValidatorEnabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect_ip', 'ip' );
		}

		/**
		 * Check if Multiple Submission Protection is enabled
		 *
		 * @return bool Whether Multiple Submission Protection is enabled
		 */
		private function isMultipleSubmissionProtectionEnabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect_avada_multiple_submissions', 'avada' );
		}

		/**
		 * Check if Time Validator is enabled
		 *
		 * @return bool Whether Time Validator is enabled
		 */
		private function isTimeValidatorEnabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect_avada_time_enable', 'avada' );
		}

		/**
		 * Check if Captcha is enabled
		 *
		 * @return boolean Whether Captcha is enabled
		 */
		public function isCaptchaEnabled() {
			return (bool) CF7Captcha::getInstance()->getSettings( 'protect_avada', 'avada' );
		}

		/**
		 * Get the Captcha method
		 *
		 * @return mixed Captcha method
		 */
		public static function getCaptchaMethod() {
			return CF7Captcha::getInstance()->getSettings( 'protect_avada_method', 'avada' );
		}
	}
}
